/*
 * Amanda, The Advanced Maryland Automatic Network Disk Archiver
 * Copyright (c) 1991 University of Maryland
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of U.M. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  U.M. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * U.M. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL U.M.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: James da Silva, Systems Design and Analysis Group
 *			   Computer Science Department
 *			   University of Maryland at College Park
 */
#include "amanda.h"

/* 

splits 'line' on spaces and returns each token in 'args'.  The number
of tokens is returned.  Too many tokens returns -1, no tokens returns
NULL.  Used in client-src/sendbackup-dump.c and sendsize-dump.c

Note: uses strdup() and strtok() calls.

Author:  John Stoffel, College Computer Center
                       Worcester Polytechnic Institute
		       john@wpi.edu
*/

int splitline(line,args)
char *line;
char *args[];
{
  char *t;
  int i;
  i = 0;

  if ((t = strtok(line," ")) != NULL) {
    args[i++] = strdup(t);
    while ((t = strtok((char *)NULL," ")) != NULL) {
      if (i >= MAXDUMPKEYS) {
	args[MAXDUMPKEYS-1] = (char *) NULL;
	return(-1);       /* too many keys */
      }
      args[i++] = strdup(t);
    }
    args[i] = (char *) NULL;
    return(i);            /* number keys found */
  }
  else {
    args[0] = (char *) NULL;
    return(0);         /* empty string passed in */
  }
}
